@@ -16,152 +16,155 @@ end
 
 local _dims = pl.Set { "width", "height", "depth" }
 
-nodefactory.box = pl.class({
-    type = "special",
-    height = nil,
-    depth = nil,
-    width = nil,
-    misfit = false,
-    explicit = false,
-    discardable = false,
-    value = nil,
-    _default_length = "width",
+nodefactory.box = pl.class()
+nodefactory.box.type = "special"
 
-    _init = function (self, spec)
-      if type(spec) == "string"
-        or type(spec) == "number"
-        or SU.type(spec) == "measurement"
-        or SU.type(spec) == "length" then
-        self[self._default_length] = SU.cast("length", spec)
-      elseif SU.type(spec) == "table" then
-        if spec._tospec then spec = spec:_tospec() end
-        for k, v in pairs(spec) do
-          self[k] = _dims[k] and SU.cast("length", v) or v
-        end
-      elseif type(spec) ~= "nil" and SU.type(spec) ~= self.type then
-        SU.error("Unimplemented, creating " .. self.type .. " node from " .. SU.type(spec), 1)
-      end
-      for dim in pairs(_dims) do
-        if not self[dim] then self[dim] = SILE.length() end
-      end
-      self["is_"..self.type] = true
-      self.is_box = self.is_hbox or self.is_vbox or self.is_zerohbox or self.is_alternative or self.is_nnode
-      self.is_zero = self.is_zerohbox or self.is_zerovglue
-      if self.is_migrating then self.is_hbox, self.is_box = true, true end
-    end,
+nodefactory.box.height = nil
+nodefactory.box.depth = nil
+nodefactory.box.width = nil
+nodefactory.box.misfit = false
+nodefactory.box.explicit = false
+nodefactory.box.discardable = false
+nodefactory.box.value = nil
+nodefactory.box._default_length = "width"
 
-    -- De-init instances by shallow copying properties and removing meta table
-    _tospec = function (self)
-      return pl.tablex.copy(self)
-    end,
-
-    tostring = function (self)
-      return  self:__tostring()
-    end,
-
-    __tostring = function (self)
-      return self.type
-    end,
-
-    __concat = function (a, b)
-      return tostring(a) .. tostring(b)
-    end,
-
-    absolute = function (self)
-      local clone = nodefactory[self.type](self:_tospec())
-      for dim in pairs(_dims) do
-        clone[dim] = self[dim]:absolute()
-      end
-      if self.nodes then
-        clone.nodes = pl.tablex.map_named_method("absolute", self.nodes)
-      end
-      return clone
-    end,
-
-    lineContribution = function (self)
-      -- Regardless of the orientations, "width" is always in the
-      -- writingDirection, and "height" is always in the "pageDirection"
-      return self.misfit and self.height or self.width
-    end,
-
-    outputYourself = function (self)
-      SU.error(self.type.." with no output routine")
-    end,
-
-    toText = function (self)
-      return self.type
-    end,
-
-    isBox = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "hbox" or self.type == "zerohbox" or self.type == "alternative" or self.type == "nnode" or self.type == "vbox"
-    end,
-
-    isNnode = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type=="nnode"
-    end,
-
-    isGlue = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "glue"
-    end,
-
-    isVglue = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "vglue"
-    end,
-
-    isZero = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "zerohbox" or self.type == "zerovglue"
-    end,
-
-    isUnshaped = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "unshaped"
-    end,
-
-    isAlternative = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "alternative"
-    end,
-
-    isVbox = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "vbox"
-    end,
-
-    isInsertion = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "insertion"
-    end,
-
-    isMigrating = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.migrating
-    end,
-
-    isPenalty = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "penalty"
-    end,
-
-    isDiscretionary = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "discretionary"
-    end,
-
-    isKern = function (self)
-      SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
-      return self.type == "kern"
+function nodefactory.box:_init (spec)
+  if type(spec) == "string"
+    or type(spec) == "number"
+    or SU.type(spec) == "measurement"
+    or SU.type(spec) == "length" then
+    self[self._default_length] = SU.cast("length", spec)
+  elseif SU.type(spec) == "table" then
+    if spec._tospec then spec = spec:_tospec() end
+    for k, v in pairs(spec) do
+      self[k] = _dims[k] and SU.cast("length", v) or v
     end
+  elseif type(spec) ~= "nil" and SU.type(spec) ~= self.type then
+    SU.error("Unimplemented, creating " .. self.type .. " node from " .. SU.type(spec), 1)
+  end
+  for dim in pairs(_dims) do
+    if not self[dim] then self[dim] = SILE.length() end
+  end
+  self["is_"..self.type] = true
+  self.is_box = self.is_hbox or self.is_vbox or self.is_zerohbox or self.is_alternative or self.is_nnode
+  self.is_zero = self.is_zerohbox or self.is_zerovglue
+  if self.is_migrating then self.is_hbox, self.is_box = true, true end
+end
 
-  })
+-- De-init instances by shallow copying properties and removing meta table
+function nodefactory.box:_tospec ()
+  return pl.tablex.copy(self)
+end
+
+function nodefactory.box:tostring ()
+  return  self:__tostring()
+end
+
+function nodefactory.box:__tostring ()
+  return self.type
+end
+
+function nodefactory.box.__concat (a, b)
+  return tostring(a) .. tostring(b)
+end
+
+function nodefactory.box:absolute ()
+  local clone = nodefactory[self.type](self:_tospec())
+  for dim in pairs(_dims) do
+    clone[dim] = self[dim]:absolute()
+  end
+  if self.nodes then
+    clone.nodes = pl.tablex.map_named_method("absolute", self.nodes)
+  end
+  return clone
+end
+
+function nodefactory.box:lineContribution ()
+  -- Regardless of the orientations, "width" is always in the
+  -- writingDirection, and "height" is always in the "pageDirection"
+  return self.misfit and self.height or self.width
+end
+
+function nodefactory.box:outputYourself ()
+  SU.error(self.type.." with no output routine")
+end
+
+function nodefactory.box:toText ()
+  return self.type
+end
+
+function nodefactory.box:isBox ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "hbox" or self.type == "zerohbox" or self.type == "alternative" or self.type == "nnode" or self.type == "vbox"
+end
+
+function nodefactory.box:isNnode ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type=="nnode"
+end
+
+function nodefactory.box:isGlue ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "glue"
+end
+
+function nodefactory.box:isVglue ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "vglue"
+end
+
+function nodefactory.box:isZero ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "zerohbox" or self.type == "zerovglue"
+end
+
+function nodefactory.box:isUnshaped ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "unshaped"
+end
+
+function nodefactory.box:isAlternative ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "alternative"
+end
+
+function nodefactory.box:isVbox ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "vbox"
+end
+
+function nodefactory.box:isInsertion ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "insertion"
+end
+
+function nodefactory.box:isMigrating ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.migrating
+end
+
+function nodefactory.box:isPenalty ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "penalty"
+end
+
+function nodefactory.box:isDiscretionary ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "discretionary"
+end
+
+function nodefactory.box:isKern ()
+  SU.warn("Deprecated function, please use boolean is_<type> property to check types", true)
+  return self.type == "kern"
+end
 
 nodefactory.hbox = pl.class(nodefactory.box)
 nodefactory.hbox.type = "hbox"
 
+function nodefactory.hbox:_init (spec)
+  nodefactory.box._init(self, spec)
+end
+
 function nodefactory.hbox:__tostring ()
   return "H<" .. tostring(self.width) .. ">^" .. tostring(self.height) .. "-" .. tostring(self.depth) .. "v"
 end
@@ -210,14 +213,27 @@ function nodefactory.nnode:absolute ()
 end
 
 function nodefactory.nnode:outputYourself (typesetter, line)
+  -- See typesetter:computeLineRatio() which implements the currently rather messy
+  -- and probably slightly dubious 'hyphenated' logic.
+  -- Example: consider the word "out-put".
+  -- The node queue therefore contains N(out)D(-)N(put) all pointing to the same
+  -- parent N(output).
   if self.parent and not self.parent.hyphenated then
+    -- When we hit N(out) and are not hyphenated, we output N(output) directly
+    -- and mark it as used, so as to skip D(-)N(put) afterwards.
+    -- I guess this was done to ensure proper kerning (vs. outputting each of
+    -- the nodes separately).
     if not self.parent.used then
       self.parent:outputYourself(typesetter, line)
     end
     self.parent.used = true
-    return
+  else
+    -- It's possible not to have a parent, e.g. N(word) without hyphenation points.
+    -- Either that, or we have a hyphenated parent but are in the case we are
+    -- outputting one of the elements e.g. N(out)D(-) [line break] N(put).
+    -- (ignoring the G(margin) nodes and potentially zerohbox nodes also on either side of the line break)
+    for _, node in ipairs(self.nodes) do node:outputYourself(typesetter, line) end
   end
-  for _, node in ipairs(self.nodes) do node:outputYourself(typesetter, line) end
 end
 
 function nodefactory.nnode:toText ()
@@ -239,7 +255,7 @@ end
 getmetatable(nodefactory.unshaped).__index = function (_, _)
   -- if k == "width" then SU.error("Can't get width of unshaped node", true) end
   -- TODO: No idea why porting to proper Penlight classes this ^^^^^^ started
-  -- killing everything. Perhaps becaus this function started working and would
+  -- killing everything. Perhaps because this function started working and would
   -- actually need to return rawget(self, k) or something?
 end
 
@@ -262,7 +278,6 @@ nodefactory.discretionary.prebreak = {}
 nodefactory.discretionary.postbreak = {}
 nodefactory.discretionary.replacement = {}
 nodefactory.discretionary.used = false
-nodefactory.discretionary.prebw = nil
 
 function nodefactory.discretionary:__tostring ()
   return "D(" .. SU.concat(self.prebreak, "") .. "|" .. SU.concat(self.postbreak, "") .."|" .. SU.concat(self.replacement, "") .. ")";
@@ -273,9 +288,27 @@ function nodefactory.discretionary:toText ()
 end
 
 function nodefactory.discretionary:outputYourself (typesetter, line)
+  -- See typesetter:computeLineRatio() which implements the currently rather
+  -- messy hyphenated checks.
+  -- Example: consider the word "out-put-ter".
+  -- The node queue contains N(out)D(-)N(put)D(-)N(ter) all pointing to the same
+  -- parent N(output), and here we hit D(-)
+
+  -- Non-hyphenated parent: when N(out) was hit, we went for outputting
+  -- the whole parent, so all other elements must now be skipped.
+  if self.parent and not self.parent.hyphenated then return end
+
+  -- It's possible not to have a parent (e.g. on a discretionary directly
+  -- added in the queue and not coming from the hyphenator logic).
+  -- Eiher that, or we have a hyphenate parent.
   if self.used then
+    -- This is the actual hyphenation point.
+    -- Skip margin glue and zero boxes.
+    -- If we then reach our discretionary, it means its the first in the line,
+    -- i.e. a postbreak. Otherwise, its a prebreak (near the end of the line,
+    -- notwithstanding glues etc.)
     local i = 1
-    while (line.nodes[i].is_glue and line.nodes[i].value == "lskip")
+    while (line.nodes[i].is_glue and line.nodes[i].value == "margin")
       or line.nodes[i].type == "zerohbox" do
       i = i + 1
     end
@@ -285,6 +318,10 @@ function nodefactory.discretionary:outputYourself (typesetter, line)
       for _, node in ipairs(self.prebreak) do node:outputYourself(typesetter, line) end
     end
   else
+    -- This is not the hyphenation point (but another discretionary in the queue)
+    -- E.g. we were in the case where we have N(out)D(-) [line break] N(out)D(-)N(ter)
+    -- and now hit the second D(-).
+    -- Unused discretionaries are obviously replaced.
     for _, node in ipairs(self.replacement) do node:outputYourself(typesetter, line) end
   end
 end
@@ -299,7 +336,7 @@ end
 function nodefactory.discretionary:postbreakWidth ()
   if self.postbw then return self.postbw end
   self.postbw = SILE.length()
-  for _, node in ipairs(self.postbreak) do self.pastbw:___add(node.width) end
+  for _, node in ipairs(self.postbreak) do self.postbw:___add(node.width) end
   return self.postbw
 end
 
@@ -328,6 +365,12 @@ function nodefactory.discretionary:replacementHeight ()
   return self.replaceh
 end
 
+function nodefactory.discretionary:replacementDepth ()
+  if self.replaced then return self.replaced end
+  self.replaced = _maxnode(self.replacement, "depth")
+  return self.replaced
+end
+
 nodefactory.alternative = pl.class(nodefactory.hbox)
 
 nodefactory.alternative.type = "alternative"
@@ -373,21 +416,24 @@ function nodefactory.glue:outputYourself (typesetter, line)
   typesetter.frame:advanceWritingDirection(outputWidth)
 end
 
+-- A hfillglue is just a glue with infinite stretch.
+-- (Convenience so callers do not have to know what infinity is.)
 nodefactory.hfillglue = pl.class(nodefactory.glue)
 function nodefactory.hfillglue:_init (spec)
-  self.width = SILE.length(0, infinity)
   self:super(spec)
+  self.width = SILE.length(self.width.length, infinity, self.width.shrink)
 end
 
--- possible bug, deprecated constructor actually used vglue as base class for this
+-- A hssglue is just a glue with infinite stretch and shrink.
+-- (Convenience so callers do not have to know what infinity is.)
 nodefactory.hssglue = pl.class(nodefactory.glue)
 function nodefactory.hssglue:_init (spec)
-  self.width = SILE.length(0, infinity, infinity)
   self:super(spec)
+  self.width = SILE.length(self.width.length, infinity, infinity)
 end
 
 nodefactory.kern = pl.class(nodefactory.glue)
-nodefactory.kern.type = "kern"
+nodefactory.kern.type = "kern" -- Perhaps some smell here, see comment on vkern
 nodefactory.kern.discardable = false
 
 function nodefactory.kern:__tostring ()
@@ -421,18 +467,34 @@ function nodefactory.vglue:unbox ()
   return { self }
 end
 
+-- A vfillglue is just a vglue with infinite stretch.
+-- (Convenience so callers do not have to know what infinity is.)
 nodefactory.vfillglue = pl.class(nodefactory.vglue)
 function nodefactory.vfillglue:_init (spec)
-  self.height = SILE.length(0, infinity)
   self:super(spec)
+  self.height = SILE.length(self.width.length, infinity, self.width.shrink)
 end
 
+-- A vssglue is just a vglue with infinite stretch and shrink.
+-- (Convenience so callers do not have to know what infinity is.)
 nodefactory.vssglue = pl.class(nodefactory.vglue)
-nodefactory.vssglue.height = SILE.length(0, infinity, infinity)
+function nodefactory.vssglue:_init (spec)
+  self:super(spec)
+  self.height = SILE.length(self.width.length, infinity, infinity)
+end
 
 nodefactory.zerovglue = pl.class(nodefactory.vglue)
 
 nodefactory.vkern = pl.class(nodefactory.vglue)
+-- FIXME TODO
+-- Here we cannot do:
+--   nodefactory.vkern.type = "vkern"
+-- It cannot be typed as "vkern" as the pagebuilder doesn't check is_vkern.
+-- So it's just a vglue currrenty, marked as not discardable...
+-- But on the other hand, nodefactory.kern is typed "kern" and is not a glue...
+-- Frankly, the discardable/explicit flags and the types are too
+-- entangled and point towards a more general design issue.
+-- N.B. this vkern node is only used in the linespacing package so far.
 nodefactory.vkern.discardable = false
 
 function nodefactory.vkern:__tostring ()
@@ -626,8 +688,6 @@ setmetatable(nodefactory, {
     __index = function (_, prop)
       if _deprecated_nodefactory[prop] then
         SU.deprecated("SILE.nodefactory." .. prop, "SILE.nodefactory." .. prop:match("n?e?w?(.*)"):lower(), "0.10.0", "0.14.0")
-        local old_constructor = _deprecated_nodefactory[prop]
-        return string.find(prop, "^new") and old_constructor or old_constructor()
       elseif type(prop) == "number" then -- luacheck: ignore 542
         -- Likely at attempt to iterate (or dump) the table, sort of safe to ignore
       else
