@@ -1,3 +1,8 @@
+local base = require("packages.base")
+
+local package = pl.class(base)
+package._name = "svg"
+
 local svg = require("svg")
 local otparser = require("core.opentype-parser")
 
@@ -28,23 +33,40 @@ local _drawSVG = function (svgdata, width, height, density, drop)
     })
 end
 
-local function registerCommands (_)
+function package:registerRawHandlers ()
 
-  SILE.registerCommand("svg", function (options, _)
-    local fn = SU.required(options, "src", "filename")
+  self:registerRawHandler("svg", function(options, content)
+    local svgdata = content[1]
     local width = options.width and SU.cast("measurement", options.width):absolute() or nil
     local height = options.height and SU.cast("measurement", options.height):absolute() or nil
     local density = options.density or 72
-    local svgfile = io.open(fn)
+    -- See issue #1375: svg.svg_to_ps() called in _drawSVG has a apparently a side effect
+    -- on the internal representation of the Lua string and corrupts it.
+    -- So as a workaround, for the original string to be able to be reused, we must get a
+    -- copy... So let's force some stupid comment concatenation here.
+    _drawSVG("<!-- copy -->"..svgdata, width, height, density)
+  end)
+
+end
+
+function package:registerCommands ()
+
+  self:registerCommand("svg", function (options, _)
+    local src = SU.required(options, "src", "filename")
+    src = SILE.resolveFile(src) or SU.error("Couldn't find file " .. src)
+    local width = options.width and SU.cast("measurement", options.width):absolute() or nil
+    local height = options.height and SU.cast("measurement", options.height):absolute() or nil
+    local density = options.density or 72
+    local svgfile = io.open(src)
     local svgdata = svgfile:read("*all")
     _drawSVG(svgdata, width, height, density)
   end)
 
-  SILE.registerCommand("include-svg-file", function (_, _)
+  self:registerCommand("include-svg-file", function (_, _)
     SU.deprecated("\\include-svg-file", "\\svg", "0.10.10", "0.11.0")
   end, "Deprecated")
 
-  SILE.registerCommand("svg-glyph", function(_, content)
+  self:registerCommand("svg-glyph", function(_, content)
     local fontoptions = SILE.font.loadDefaults({})
     local items = SILE.shaper:shapeToken(content[1], fontoptions)
     local face = SILE.shaper.getFace(fontoptions)
@@ -60,31 +82,24 @@ local function registerCommands (_)
 
 end
 
-return {
-  registerCommands = registerCommands,
-  documentation = [[
+package.documentation = [[
 \begin{document}
 This package provides two commands.
 
 The first is \autodoc:command{\svg[src=<file>]}.
-This loads and parses an SVG file and attempts to render it in the current
-document. Optional \autodoc:parameter{width} or \autodoc:parameter{height}
-options will scale the SVG canvas to the given size calculated at a given
-\autodoc:parameter{density} option (which defaults to 72 ppi). For
-example, the command \autodoc:command{\svg[src=packages/svg/smiley.svg,height=12pt]}
-produces the following:
+This loads and parses an SVG file and attempts to render it in the current document.
+Optional \autodoc:parameter{width} or \autodoc:parameter{height} options will scale the SVG canvas to the given size calculated at a given \autodoc:parameter{density} option (which defaults to 72 ppi).
+For example, the command \autodoc:command{\svg[src=packages/svg/smiley.svg,height=12pt]} produces the following:
 
 \svg[src=packages/svg/smiley.svg,height=12pt]
 
-The second is a more experimental \autodoc:command{\svg-glyph}. When the current font is
-set to an SVG font, SILE does not currently render the SVG glyphs
-automatically. This command is intended to be used as a means of eventually
-implementing SVG fonts; it retrieves the SVG glyph provided and renders it.
+The second is a more experimental \autodoc:command{\svg-glyph}.
+When the current font is set to an SVG font, SILE does not currently render the SVG glyphs automatically.
+This command is intended to be used as a means of eventually implementing SVG fonts; it retrieves the SVG glyph provided and renders it.
 
-In both cases the rendering is done with our own SVG drawing library; it is currently
-very minimal, only handling lines, curves, strokes and fills. For a fuller
-implementation, consider using a \autodoc:package{converters} registration to render
-your SVG file to PDF and include it on the fly.
+In both cases the rendering is done with our own SVG drawing library; it is currently very minimal, only handling lines, curves, strokes and fills.
+For a fuller implementation, consider using a \autodoc:package{converters} registration to render your SVG file to PDF and include it on the fly.
 \end{document}
 ]]
-}
+
+return package
