@@ -1,25 +1,21 @@
+local base = require("packages.base")
+
+local package = pl.class(base)
+package._name = "tate"
+
 SILE.tateFramePrototype = pl.class(SILE.framePrototype)
 SILE.tateFramePrototype.direction = "TTB-RTL"
 
+
 SILE.tateFramePrototype.enterHooks = {
     function (_, typesetter)
-      typesetter.old_leadingFor = typesetter.leadingFor
-      typesetter.old_breakIntoLines = typesetter.breakIntoLines
-      typesetter.leadingFor = function(_, v)
-        v.height = SILE.length("1zw"):absolute()
-        local bls = SILE.settings:get("document.baselineskip")
-        local d = bls.height:absolute() - v.height
-        local len = SILE.length(d.length, bls.height.stretch, bls.height.shrink)
-        return SILE.nodefactory.vglue({height = len})
-      end
-      typesetter.breakIntoLines = require("packages.break-firstfit").exports.breakIntoLines
+      SILE.typesetters.tate:cast(typesetter)
     end
   }
 
 SILE.tateFramePrototype.leaveHooks = {
     function (_, typesetter)
-      typesetter.leadingFor = typesetter.old_leadingFor
-      typesetter.breakIntoLines = typesetter.old_breakIntoLines
+      SILE.typesetters.base:cast(typesetter)
     end
   }
 
@@ -51,30 +47,30 @@ local outputTateChuYoko = function (self, typesetter, line)
 
 end
 
-local function registerCommands (class)
+function package:registerCommands ()
 
-  SILE.registerCommand("tate-frame", function (options, _)
+  self:registerCommand("tate-frame", function (options, _)
     SILE.documentState.thisPageTemplate.frames[options.id] = SILE.newTateFrame(options)
   end, "Declares (or re-declares) a frame on this page.")
 
   -- Eventually will be automatically called by script detection, but for now
   -- called manually
-  SILE.registerCommand("latin-in-tate", function (_, content)
+  self:registerCommand("latin-in-tate", function (_, content)
     if SILE.typesetter.frame:writingDirection() ~= "TTB" then
       return SILE.process(content)
     end
     local nodes
     local oldT = SILE.typesetter
     local prevDirection = oldT.frame.direction
-    class:loadPackage("rotate")
+    self:loadPackage("rotate")
     SILE.settings:temporarily(function()
-      local latinTypesetter = pl.class(SILE.defaultTypesetter)
+      local latinTypesetter = pl.class(SILE.typesetters.base)
       local dummyFrame = pl.class(SILE.framePrototype)
-      dummyFrame.init = function (self)
-        self.state = {}
+      dummyFrame.init = function (f)
+        f.state = {}
       end
-      latinTypesetter.initFrame = function (self, frame)
-        self.frame = frame
+      latinTypesetter.initFrame = function (typesetter, frame)
+        typesetter.frame = frame
       end
       local frame = dummyFrame({}, true)
       SILE.typesetter = latinTypesetter(frame)
@@ -93,23 +89,22 @@ local function registerCommands (class)
       if SILE.typesetter.frame:writingDirection() ~= "TTB" or nodes[i].is_glue then
         SILE.typesetter:pushHorizontal(nodes[i])
       elseif nodes[i]:lineContribution():tonumber() > 0 then
-        SILE.call("hbox", {}, function ()
+        local hbox = SILE.call("hbox", {}, function ()
           SILE.typesetter:pushHorizontal(nodes[i])
         end)
-        local n = SILE.typesetter.state.nodes[#SILE.typesetter.state.nodes]
         -- Turn off all complex flags.
-        for j = 1,#(n.value) do
-          for k = 1,#(n.value[j].nodes) do
-            n.value[j].nodes[k].value.complex = false
+        for j = 1, #(hbox.value) do
+          for k = 1, #(hbox.value[j].nodes) do
+            hbox.value[j].nodes[k].value.complex = false
           end
         end
-        n.oldOutputYourself = n.outputYourself
-        n.outputYourself = outputLatinInTate
+        hbox.oldOutputYourself = hbox.outputYourself
+        hbox.outputYourself = outputLatinInTate
       end
     end
   end, "Typeset rotated Western text in vertical Japanese")
 
-  SILE.registerCommand("tate-chu-yoko", function (_, content)
+  self:registerCommand("tate-chu-yoko", function (_, content)
     if SILE.typesetter.frame:writingDirection() ~= "TTB" then return SILE.process(content) end
     -- SILE.typesetter:pushGlue({
     --   width = SILE.length.new({length = SILE.toPoints("0.5zw"),
@@ -121,11 +116,10 @@ local function registerCommands (class)
       SILE.settings:set("document.language", "und")
       SILE.settings:set("font.direction", "LTR")
       SILE.call("rotate",{angle =-90}, function ()
-        SILE.call("hbox", {}, content)
-        local n = SILE.typesetter.state.nodes[#SILE.typesetter.state.nodes]
-        n.misfit = true
-        n.oldOutputYourself = n.outputYourself
-        n.outputYourself = outputTateChuYoko
+        local hbox = SILE.call("hbox", {}, content)
+        hbox.misfit = true
+        hbox.oldOutputYourself = hbox.outputYourself
+        hbox.outputYourself = outputTateChuYoko
       end)
 
     end)
@@ -140,16 +134,11 @@ local function registerCommands (class)
 
 end
 
-return {
-  registerCommands = registerCommands,
-  documentation = [[
+package.documentation = [[
 \begin{document}
 The \autodoc:package{tate} package provides support for Japanese vertical typesetting.
-It allows for the definition of vertical-oriented frames, as well
-as for two specific typesetting techniques required in vertical
-documents: \autodoc:command{\latin-in-tate} typesets its content as Latin
-text rotated 90 degrees, and \autodoc:command{\tate-chu-yoko} places (Latin)
-text horizontally within a single grid-square of the vertical \em{hanmen}.
+It allows for the definition of vertical-oriented frames, as well as for two specific typesetting techniques required in vertical documents: \autodoc:command{\latin-in-tate} typesets its content as Latin text rotated 90 degrees, and \autodoc:command{\tate-chu-yoko} places (Latin) text horizontally within a single grid-square of the vertical \em{hanmen}.
 \end{document}
 ]]
-}
+
+return package
